SuperStrict

Rem
bbdoc:	CScanner
EndRem

Module mac.scanner
ModuleInfo "Version: 1.00"
ModuleInfo "Author:  Joahnnes Wolf"
ModuleInfo "License: Public domain"

Import BRL.Blitz

Rem
bbdoc: Scanning Type
about: Contains all scann Methods
End Rem
Type CScanner
	
	'FIELDS

	Field _inputString	:String
	
	Field _endPos			:Int
	Field _pos				:Int
	
	Field _caseSensitive	:Byte
	Field _skippChars		:String
	
	Field _atEnd			:Byte

	'FUNCTIONS
	
	Function Create:CScanner( inputString:String )
		Local scanner:CScanner = New CScanner
			scanner._inputString = inputString
			scanner._pos = 0
			scanner._endPos = inputString.length
			scanner._caseSensitive = False
			scanner._skippChars = " \n"
			scanner._atEnd = False
		Return scanner
	EndFunction
	
	'METHODS
	
	Method setString( inputString:String )
		Self._inputString = inputString
	EndMethod
	
	Method getString:String()
		Return Self._inputString
	EndMethod

	Method setPosition( pos:Int )
		Self._pos = pos
	EndMethod
	
	Method getPosition:Int()
		Return Self._pos
	EndMethod
	
	Method setCaseSensitive( caseSensitive:Byte )
		Self._caseSensitive = caseSensitive
	EndMethod
	
	Method getCaseSensitive:Byte()
		Return Self._caseSensitive
	EndMethod
	
	Method setSkippChars( skippChars:String )
		Self._skippChars = skippChars
	EndMethod
	
	Method getSkippChars:String()
		Return Self._skippChars
	EndMethod
	
	Method isAtEnd:Byte()
		Return Self._atEnd
	EndMethod
	
		'scanning
	
	Method scanInt:Byte(_in:Int Var)
		If Not Self._atEnd
			Local _string:String = Self._inputString
			If (__isNumber(Chr(_string[Self._pos])))
				Local _output:String
				While(__isNumber(Chr(_string[Self._pos])))
					_output :+ Chr(_string[Self._pos])
					If Self._pos < Self._endPos-1 
						Self._pos :+ 1
					Else
						Self._atEnd = True
						Exit
					EndIf
				Wend
				Self.__skippCharacters()
				_in = Int(_output)
				Return True
			Else
				Return False
			EndIf
		EndIf
	EndMethod
	
	Method scanFloat:Byte(_in:Float Var)
		Local _string:String = Self._inputString
		If Not Self._atEnd
			If (__isNumber(Chr(_string[Self._pos])))
				Local _output:String
				While(__isNumber(Chr(_string[Self._pos])))
					_output :+ Chr(_string[Self._pos])
					If Self._pos < Self._endPos-1 
						Self._pos :+ 1
					Else
						Self._atEnd = True
						Exit
					EndIf
				Wend
				Self.__skippCharacters()
				_in = Float(_output)
				Return True
			Else
				Return False
			EndIf
		EndIf
	EndMethod
	
	Method scanDouble:Byte(_in:Double Var)
		If Not Self._atEnd
			Local _string:String = Self._inputString
			If (__isNumber(Chr(_string[Self._pos])))
				Local _output:String
				While(__isNumber(Chr(_string[Self._pos])))
					_output :+ Chr(_string[Self._pos])
					If Self._pos < Self._endPos-1 
						Self._pos :+ 1
					Else
						Self._atEnd = True
						Exit
					EndIf
				Wend
				Self.__skippCharacters()
				_in = Double(_output)
				Return True
			Else
				Return False
			EndIf
		EndIf
	EndMethod
	
	Method scanString:Byte(_in:String)
		If Not Self._atEnd
			Local _string:String = Self._inputString
			If Self._pos + _in.length <= Self._endPos
				Local _startPos:Int = Self._pos
				Local i:Int
				For i = Self._pos To Self._pos + _in.length - 1
					If _string[i] = _in[i - _startPos]
						'nothing
					Else
						Return False
					EndIf
				Next
				If i >= Self._endPos
					Self._atEnd = True
				EndIf
				Self._pos = i
				Self.__skippCharacters()
				Return True
			Else
				Return False
			EndIf
		EndIf
	EndMethod
	
	Method scanStringIntoString:Byte(_in:String, _out:String Var)
		If Not Self._atEnd
			Local _string:String = Self._inputString
			If Self._pos + _in.length <= Self._endPos
				Local _output:String
				Local _startPos:Int = Self._pos
				Local i:Int
				For i = Self._pos To Self._pos + _in.length - 1
					If _string[i] = _in[i - _startPos]
						_output :+ Chr(_string[i])
					Else
						Return False
					EndIf
				Next
				If i >= Self._endPos
					Self._atEnd = True
				EndIf
				Self._pos = i
				_out = _output
				Self.__skippCharacters()
				Return True
			Else
				Return False
			EndIf
		EndIf
	EndMethod
	
	Method scanUpToString:Byte( _in:String )
		If Not Self._atEnd
			Local _string:String = Self._inputString
			Local _startPos:Int = Self._pos
			If Self._pos + _in.length <= Self._endPos
				Local i:Int
				For i = Self._pos To Self._endPos - _in.length
					Self._pos = i
					If Not Self.scanString(_in)
						'nothing
					Else
						If Self._pos >= Self._endPos 
							Self._atEnd = True
						EndIf
						Self.__skippCharacters()
						Return True
					EndIf
				Next
				Self._pos = _startPos
				Return False
			Else
				Return False
			EndIf
		EndIf
	EndMethod
	
	Method scanUpToStringIntoString:Byte( _in:String, _out:String Var )
		If Not Self._atEnd
			Local _string:String = Self._inputString
			Local _output:String
			Local _startPos:Int = Self._pos
			If Self._pos + _in.length <= Self._endPos
				Local i:Int
				For i = Self._pos To Self._endPos - _in.length
					Self._pos = i
					If Not Self.scanString(_in)
						_output :+ Chr(_string[i])
					Else
						If Self._pos >= Self._endPos 
							Self._atEnd = True
						EndIf
						_out = _output
						Self.__skippCharacters()
						Return True
					EndIf
				Next
				Self._pos = _startPos
				Return False
			Else
				Return False
			EndIf
		EndIf
	EndMethod
	
		'private
		
	Method __skippCharacters()
		If Not Self._atEnd 
			While(__isSkippChar(Chr(Self._inputString[Self._pos]),Self._skippChars))
				Self._pos :+ 1
				If Self._pos = Self._endPos
					Self._atEnd = True
					Return
				EndIf 
			Wend
		EndIf
	EndMethod

EndType

'FUNCTIONS

Function __isNumber:Byte( char:String )
	Local asci:Int = Asc(char)
	If asci >= 48 And asci <= 57 Or asci = 46
		Return True
	Else
		Return False
	EndIf
EndFunction

Function __isSkippChar:Byte( char:String, skipp:String )
	Local asci:Int = Asc(char)
	For Local i:Int = 0 To skipp.length-1
		If asci = skipp[i]
			Return True
		EndIf
	Next
	Return False
EndFunction
